/************************************************************
 **
 ** pic16op.h : 16 bit PIC opcode declarations
 **
 ** Copyright (c) 2007, Kyle A. York
 ** All rights reserved
 **
 ************************************************************/
#ifndef pic16op_h__
#define pic16op_h__

/*
 * d = 0 (W) 1 (F)
 * a = 0 (access) 1 (BSR)
 * b = bit (0 <= b <= 7)
 * r = relative offset -128 <= r <= 127
 * R = relative offset -1024 <= R <M= 1023
 * s = 0 (no update) 1 (save W/Status/BSR)
 * k = absolute address or value
 * x = doesn't matter
 * n = 0 (no change) 1 (post incr) 2 (post decr) 3 (pre incr)
 */
typedef enum {
  pic16_opcode_org,
  pic16_opcode_end,
  pic16_opcode_none,
  /* literal instructions on W */
  pic16_opcode_sublw,  /* 0000 1000 kkkk kkkk */
  pic16_opcode_iorlw,  /* 0000 1001 kkkk kkkk */
  pic16_opcode_xorlw,  /* 0000 1010 kkkk kkkk */
  pic16_opcode_andlw,  /* 0000 1011 kkkk kkkk */
  pic16_opcode_retlw,  /* 0000 1100 kkkk kkkk */
  pic16_opcode_mullw,  /* 0000 1101 kkkk kkkk */
  pic16_opcode_movlw,  /* 0000 1110 kkkk kkkk */
  pic16_opcode_addlw,  /* 0000 1111 kkkk kkkk */
  /* file register instructions w/ dst */
  pic16_opcode_decf,   /* 0000 01da ffff ffff */
  pic16_opcode_iorwf,  /* 0001 00da ffff ffff */
  pic16_opcode_andwf,  /* 0001 01da ffff ffff */
  pic16_opcode_xorwf   /* 0001 10da ffff ffff */
  pic16_opcode_comf,   /* 0001 11da ffff ffff */
  pic16_opcode_addwfc, /* 0010 00da ffff ffff */
  pic16_opcode_addwf,  /* 0010 01da ffff ffff */
  pic16_opcode_incf,   /* 0010 10da ffff ffff */
  pic16_opcode_decfsz, /* 0010 11da ffff ffff */
  pic16_opcode_rrcf,   /* 0011 00da ffff ffff */
  pic16_opcode_rlcf,   /* 0011 01da ffff ffff */
  pic16_opcode_swapf,  /* 0011 10da ffff ffff */
  pic16_opcode_incfsz, /* 0011 11da ffff ffff */
  pic16_opcode_rrncf,  /* 0100 00da ffff ffff */
  pic16_opcode_rlncf,  /* 0100 01da ffff ffff */
  pic16_opcode_infsnz, /* 0100 10da ffff ffff */
  pic16_opcode_dcfsnz, /* 0100 11da ffff ffff */
  pic16_opcode_movf,   /* 0101 00da ffff ffff */
  pic16_opcode_subfwb, /* 0101 01da ffff ffff */
  pic16_opcode_subwfb, /* 0101 10da ffff ffff */
  pic16_opcode_subwf,  /* 0101 11da ffff ffff */
  /* file register instructions w/o dst */
  pic16_opcode_mulwf,  /* 0000 001a ffff ffff */
  pic16_opcode_cpfslt, /* 0110 000a ffff ffff */
  pic16_opcode_cpfseq, /* 0110 001a ffff ffff */
  pic16_opcode_cpfsgt, /* 0110 010a ffff ffff */
  pic16_opcode_tstfsz, /* 0110 011a ffff ffff */
  pic16_opcode_setf,   /* 0110 100a ffff ffff */
  pic16_opcode_clrf,   /* 0110 101a ffff ffff */
  pic16_opcode_negf,   /* 0110 110a ffff ffff */
  pic16_opcode_movwf,  /* 0110 111a ffff ffff */
  /* bit instructions */
  pic16_opcode_btg,    /* 0111 bbba ffff ffff */
  pic16_opcode_bsf,    /* 1000 bbba ffff ffff */
  pic16_opcode_bcf,    /* 1001 bbba ffff ffff */
  pic16_opcode_btfss,  /* 1010 bbba ffff ffff */
  pic16_opcode_btfsc,  /* 1011 bbba ffff ffff */
  /* relative branch instructions -128...127 */
  pic16_opcode_bz,     /* 1110 0000 rrrr rrrr */
  pic16_opcode_bnz,    /* 1110 0001 rrrr rrrr */
  pic16_opcode_bc,     /* 1110 0010 rrrr rrrr */
  pic16_opcode_bnc,    /* 1110 0011 rrrr rrrr */
  pic16_opcode_bov,    /* 1110 0100 rrrr rrrr */
  pic16_opcode_bnov,   /* 1110 0101 rrrr rrrr */
  pic16_opcode_bn,     /* 1110 0110 rrrr rrrr */
  pic16_opcode_bnn,    /* 1110 0111 rrrr rrrr */
  /* relative branch instruction */
  pic16_opcode_bra,    /* 1101 0RRR RRRR RRRR */
  pic16_opcode_rcall,  /* 1101 1RRR RRRR RRRR */
  /* long branch instruction */
  pic16_opcode_call,   /* 1110 110s kkkk kkkk ( 7...0) */
                       /* 1111 kkkk kkkk kkkk (19...8) */
  pic16_opcode_goto,   /* 1110 1111 kkkk kkkk ( 7..0) */
                       /* 1111 kkkk kkkk kkkk (19..8) */
  /* no operand instruction */
  pic16_opcode_nop,    /* 0000 0000 0000 0000 */
                       /* 1111 xxxx xxxx xxxx */
  pic16_opcode_sleep,  /* 0000 0000 0000 0011 */
  pic16_opcode_clrwdt, /* 0000 0000 0000 0100 */
  pic16_opcode_push,   /* 0000 0000 0000 0101 */
  pic16_opcode_pop,    /* 0000 0000 0000 0110 */
  pic16_opcode_daw,    /* 0000 0000 0000 0111 */
  pic16_opcode_reset,  /* 0000 0000 1111 1111 */
  /* misc. instructions */
  pic16_opcode_lfsr,   /* 1110 1110 00ff kkkk (11..8) */
                       /* 1111 0000 kkkk kkkk ( 7..0) */
  pic16_opcode_movff,  /* 1100 ffff ffff ffff (src) */
                       /* 1111 ffff ffff ffff (dst) */
  pic16_opcode_movlb,  /* 0000 0001 kkkk kkkk */
  pic16_opcode_retfie, /* 0000 0000 0001 000s */
  pic16_opcode_return, /* 0000 0000 0001 001s */
  pic16_opcode_tblrd,  /* 0000 0000 0000 10nn */
  pic16_opcode_tblwt,  /* 0000 0000 0000 11nn */
} pic16_opcode_t;


#endif /* pic16op_h__ */

